#comparison of white et al results

#http://www.nicebread.de/exploring-the-robustness-of-bayes-factors-a-convenient-plotting-function-2/ 
## This source code is licensed under the FreeBSD license
## (c) 2013 Felix Schönbrodt

#' @title Plots a comparison of a sequence of priors for t test Bayes factors
#'
#' @details
#' 
#'
#' @param ts A vector of t values
#' @param ns A vector of corresponding sample sizes
#' @param rs The sequence of rs that should be tested. r should run up to 2 (higher values are implausible; E.-J. Wagenmakers, personal communication, Aug 22, 2013)
#' @param labels Names for the studies (displayed in the facet headings)
#' @param dots Values of r's which should be marked with a red dot
#' @param plot If TRUE, a ggplot is returned. If false, a data frame with the computed Bayes factors is returned
#' @param sides If set to "two" (default), a two-sided Bayes factor is computed. If set to "one", a one-sided Bayes factor is computed. In this case, it is assumed that positive t values correspond to results in the predicted direction and negative t values to results in the unpredicted direction. For details, see Wagenmakers, E. J., & Morey, R. D. (2013). Simple relation between one-sided and two-sided Bayesian point-null hypothesis tests.
#' @param nrow Number of rows of the faceted plot.
#' @param forH1 Defines the direction of the BF. If forH1 is TRUE, BF > 1 speak in favor of H1 (i.e., the quotient is defined as H1/H0). If forH1 is FALSE, it's the reverse direction.
#'
#' @references

# Bem data one-sided
BFrobustplot(
  ts=c(2.18, -2.39, -4.58, 4.78, -1.92, 4.51, -3.44, -6.08),
  ns=c(49, 49, 19, 19, 40, 40, 11, 11),
  dots=1, sides="two", forH1 = FALSE, labels=c("Experiment 1", "Experiment 2", "Experiment 3", "Experiment 4", "Experiment 5", "Experiment 6", "Experiment 7", "Experiment 8"),)


  
#http://www.nicebread.de/exploring-the-robustness-of-bayes-factors-a-convenient-plotting-function-2/
#' Rouder, J. N., Speckman, P. L., Sun, D., Morey, R. D., & Iverson, G. (2009). Bayesian t-tests for accepting and rejecting the null hypothesis. Psychonomic Bulletin and Review, 16, 225-237.
#' Wagenmakers, E.-J., & Morey, R. D. (2013). Simple relation between one-sided and two-sided Bayesian point-null hypothesis tests. Manuscript submitted for publication
#' Wagenmakers, E.-J., Wetzels, R., Borsboom, D., Kievit, R. & van der Maas, H. L. J. (2011). Yes, psychologists must change the way they analyze their data: Clarifications for Bem, Utts, & Johnson (2011)

BFrobustplot <- function(
  ts, ns, rs=seq(0, 2, length.out=200), dots=1, plot=TRUE, 
  labels=c(), sides="two", nrow=2, xticks=3, forH1=TRUE) 
{
  library(BayesFactor)
  
  # compute one-sided p-values from ts and ns
  ps <- pt(ts, df=ns-1, lower.tail = FALSE)   # one-sided test
  
  # add the dots location to the sequences of r's
  rs <- c(rs, dots)
  
  res <- data.frame()
  for (r in rs) {
    
    # first: calculate two-sided BF
    B_e0 <- c()
    for (i in 1:length(ts)) 
      B_e0 <- c(B_e0, exp(ttest.tstat(t = ts[i], n1 = ns[i], rscale=r)$bf))
    
    # second: calculate one-sided BF
    B_r0 <- c()
    for (i in 1:length(ts)) {
      if (ts[i] > 0) {
        # correct direction
        B_r0 <- c(B_r0, (2 - 2*ps[i])*B_e0[i])
      } else {
        # wrong direction
        B_r0 <- c(B_r0, (1 - ps[i])*2*B_e0[i])
      }
    }
    
    res0 <- data.frame(t=ts, n=ns, BF_two=B_e0, BF_one=B_r0, r=r)
    if (length(labels) > 0) {
      res0$labels <- labels
      res0$heading <- factor(1:length(labels), labels=paste0(labels, "\n(t = ", ts, ", df = ", ns-1, ")"), ordered=TRUE)
    } else {
      res0$heading <- factor(1:length(ts), labels=paste0("t = ", ts, ", df = ", ns-1), ordered=TRUE)
    }
    res <- rbind(res, res0)
  }
  
  # define the measure to be plotted: one- or two-sided?
  res$BF <- res[, paste0("BF_", sides)]
  
  
  # Flip BF if requested
  if (forH1 == FALSE) {
    res$BF <- 1/res$BF
  }
  
  
  if (plot==TRUE) {
    library(ggplot2)
    p1 <- ggplot(res, aes(x=r, y=log(BF))) + geom_line() + facet_wrap(~heading, nrow=nrow) + theme_bw() + ylab("log(BF)")
    p1 <- p1 + geom_hline(yintercept=c(c(-log(c(30, 10, 3)), log(c(3, 10, 30)))), linetype="dotted", color="darkgrey")
    p1 <- p1 + geom_hline(yintercept=log(1), linetype="dashed", color="darkgreen")
    
    # add the dots
    p1 <- p1 + geom_point(data=res[res$r %in% dots,], aes(x=r, y=log(BF)), color="red", size=2)
    
    # add annotation
    p1 <- p1 + annotate("text", x=max(rs)*1.8, y=-2.85, label=paste0("Strong~H[", ifelse(forH1==TRUE,0,1), "]"), hjust=1, vjust=.5, size=3, color="black", parse=TRUE)
    p1 <- p1 + annotate("text", x=max(rs)*1.8, y=-1.7 , label=paste0("Moderate~H[", ifelse(forH1==TRUE,0,1), "]"), hjust=1, vjust=.5, size=3, color="black", parse=TRUE)
    p1 <- p1 + annotate("text", x=max(rs)*1.8, y=-.55 , label=paste0("Anectodal~H[", ifelse(forH1==TRUE,0,1), "]"), hjust=1, vjust=.5, size=3, color="black", parse=TRUE)
    p1 <- p1 + annotate("text", x=max(rs)*1.8, y=2.86 , label=paste0("Strong~H[", ifelse(forH1==TRUE,1,0), "]"), hjust=1, vjust=.5, size=3, color="black", parse=TRUE)
    p1 <- p1 + annotate("text", x=max(rs)*1.8, y=1.7  , label=paste0("Moderate~H[", ifelse(forH1==TRUE,1,0), "]"), hjust=1, vjust=.5, size=3, color="black", parse=TRUE)
    p1 <- p1 + annotate("text", x=max(rs)*1.8, y=.55  , label=paste0("Anectodal~H[", ifelse(forH1==TRUE,1,0), "]"), hjust=1, vjust=.5, vjust=.5, size=3, color="black", parse=TRUE)
    
    # set scale ticks
    p1 <- p1 + scale_y_continuous(breaks=c(c(-log(c(30, 10, 3)), 0, log(c(3, 10, 30)))), labels=c("-log(30)", "-log(10)", "-log(3)", "log(1)", "log(3)", "log(10)", "log(30)"))
    p1 <- p1 + scale_x_continuous(breaks=seq(min(rs), max(rs), length.out=xticks))
    
    return(p1)
  } else {
    return(res)
  }
}

